#include <collections/vec.hpp>
#include <option.hpp>
#include <result.hpp>
#include <stl.hpp>

template <typename T>
Option<T*> Vec<T>::nth(const size_t n) {
    if (this->count <= n) {
        return Option<T*>();
    }
    return Option<T*>(this->items[n]);
}

template <typename T>
Option<const T*> Vec<T>::nth(const size_t n) const {
    if (this->count <= n) {
        return Option<const T*>();
    }
    return Option<const T*>(this->items[n]);
}

template <typename T>
Option<T> Vec<T>::pop() {
    if (this->count == 0) {
        return Option<T>();
    }

    T popped = stl::move(this->items[--this->count]);
    return Option<T>(popped);
}

template <typename T>
void Vec<T>::push(const T& val) {
    if (this->capacity <= this->count) {
        this->reserve(this->capacity * 2);
    }
    this->items[this->count++] = stl::move(val);
}

template <typename T>
void Vec<T>::push(T&& val) {}

template <typename T>
Result<void, const char*> Vec<T>::reserve(size_t new_cap) {
    if (new_cap <= this->capacity) {
        return Result<void, const char*>::err("New capacity is lower than current capacity");
    }

    T* new_data = (T*)(::operator new(sizeof(T) * new_cap));
    for (size_t i = 0; i < this->count; ++i) {
        new (new_data + i) T(stl::move(this->items[i]));
    }

    for (size_t i = 0; i < this->count; ++i) {
        this->items[i].~T();
    }

    ::operator delete(this->items);
    this->items = new_data;
    this->capacity = new_cap;
    return Result<void, const char*>::ok();
}

template <typename T>
void Vec<T>::clear() {}