
#include <option.hpp>
#include <stl.hpp>

template <typename T>
Option<T>::Option(const T& val) : has_value(true) {
    new (storage) T(val);
}

template <typename T>
Option<T>::Option(T&& val) : has_value(true) {
    new (storage) T(stl::move(val));
}

template <typename T>
Option<T>::~Option() {
    if (has_value) {
        reinterpret_cast<T*>(storage)->~T();
    }
}

template <typename T>
bool Option<T>::is_some() const {
    return has_value;
}

template <typename T>
bool Option<T>::is_none() const {
    return !has_value;
}

template <typename T>
T& Option<T>::unwrap() {
    if (!has_value) {
        __builtin_trap(); // or throw or panic 0w0
    }
    return *reinterpret_cast<T*>(storage);
}

template <typename T>
const T& Option<T>::unwrap() const {
    if (!has_value) {
        __builtin_trap();
    }
    return *reinterpret_cast<const T*>(storage);
}

template <typename T>
T* Option<T>::operator->() {
    return reinterpret_cast<T*>(storage);
}

template <typename T>
T& Option<T>::operator*() {
    return *reinterpret_cast<T*>(storage);
}