#include <result.hpp>

template <typename T, typename E>
Result<T, E> Result<T, E>::ok(const T& val) {
    Result r;
    new (r.ok_data) T(val);
    return r;
}

template <typename T, typename E>
Result<T, E> Result<T, E>::ok(T&& val) {
    Result r;
    new (r.ok_data) T(stl::move(val));
    return r;
}

template <typename T, typename E>
Result<T, E> Result<T, E>::err(const E& err) {
    Result r;
    r.is_okay = false;
    new (r.err_data) E(err);
    return r;
}

template <typename T, typename E>
Result<T, E> Result<T, E>::err(E&& err) {
    Result r;
    r.is_okay = false;
    new (r.err_data) E(stl::move(err));
    return r;
}

template <typename T, typename E>
Result<T, E>::~Result() {
    if (is_okay) {
        reinterpret_cast<T*>(ok_data)->~T();
    } else {
        reinterpret_cast<E*>(err_data)->~E();
    }
}

template <typename T, typename E>
bool Result<T, E>::is_ok() const {
    return is_okay;
}

template <typename T, typename E>
bool Result<T, E>::is_err() const {
    return !is_okay;
}

template <typename T, typename E>
T& Result<T, E>::unwrap() {
    if (!is_okay) {
        __builtin_trap();
    }
    return *reinterpret_cast<T*>(ok_data);
}

template <typename T, typename E>
E& Result<T, E>::unwrap_err() {
    if (is_okay) {
        __builtin_trap();
    }
    return *reinterpret_cast<E*>(err_data);
}